/*********************************************************************
   Sonic + Light + Bluetooth
*********************************************************************/
//For Distance Sensor
#include <NewPing.h>

#define TRIGGER_PIN  2  // Arduino pin tied to trigger pin on the ultrasonic sensor.
#define ECHO_PIN     5  // Arduino pin tied to echo pin on the ultrasonic sensor.
#define MAX_DISTANCE 200 // Maximum distance we want to ping for (in centimeters). Maximum sensor distance is rated at 400-500cm.

//For Servo Motor
#include <Servo.h>


//For LED Ring
#include <Adafruit_NeoPixel.h>
#ifdef __AVR__
  #include <avr/power.h>
#endif

#define LED 9 //Controls LED ring

//For Bluetooth Shield
#include <string.h>
#include <Arduino.h>
#include <SPI.h>

#if not defined (_VARIANT_ARDUINO_DUE_X_) && not defined (_VARIANT_ARDUINO_ZERO_)
  #include <SoftwareSerial.h>
#endif

#include "Adafruit_BLE.h"
#include "Adafruit_BluefruitLE_SPI.h"
#include "Adafruit_BluefruitLE_UART.h"

#include "BluefruitConfig.h"


/*=========================================================================
    APPLICATION SETTINGS

    FACTORYRESET_ENABLE       Perform a factory reset on the BLE sheild when 
                             running this sketch
    
    MODE_LED_BEHAVIOUR        LED activity, valid options are
                              "DISABLE" or "MODE" or "BLEUART" or
                              "HWUART"  or "SPI"  or "MANUAL"

    ROTATE                    Set degree rotation of flusher arm
    -----------------------------------------------------------------------*/
    #define FACTORYRESET_ENABLE         1
    #define MODE_LED_BEHAVIOUR          "MODE"
    #define ROTATE                      55//degrees 
/*=========================================================================*/

//Initialize Servo Motor Object and it's starting position
Servo myservo;
int pos = 180; //NOTE: Will move servo to this position and may need to be adjusted for other toitlets

//Initialize LED ring Object 
Adafruit_NeoPixel ring = Adafruit_NeoPixel(60, LED, NEO_GRB + NEO_KHZ800);

//Initialize Distance Sensor
NewPing sonar(TRIGGER_PIN, ECHO_PIN, MAX_DISTANCE);


/*hardware SPI, using SCK/MOSI/MISO hardware SPI pins and then user selected CS/IRQ/RST */
Adafruit_BluefruitLE_SPI ble(BLUEFRUIT_SPI_CS, BLUEFRUIT_SPI_IRQ, BLUEFRUIT_SPI_RST);


//Makes reporting Error messages easier to include
void error(const __FlashStringHelper*err) {
  Serial.println(err);
  while (1);
}

// function prototypes over in packetparser.cpp
uint8_t readPacket(Adafruit_BLE *ble, uint16_t timeout);
float parsefloat(uint8_t *buffer);
void printHex(const uint8_t * data, const uint32_t numBytes);

// the packet buffer
extern uint8_t packetbuffer[];


void setup(void)
{
  myservo.attach(3);  // attaches the servo on pin 3 to the servo object  
  myservo.write(0);  //Set intitial position
  myservo.detach(); //detach to avoid random servo movements

  ring.begin();
  ring.show(); // Initialize all pixels to 'off'

  
  while (!Serial);  // required for Flora & Micro
  delay(500);

  Serial.begin(115200);

  /* Initialise the module */
  Serial.print(F("Initialising the Bluefruit LE module: "));

  if ( !ble.begin(VERBOSE_MODE) )
  {
    error(F("Couldn't find Bluefruit, make sure it's in CoMmanD mode & check wiring?"));
  }
  
  Serial.println( F("OK!") );

  if ( FACTORYRESET_ENABLE )
  {
    /* Perform a factory reset to make sure everything is in a known state */
    Serial.println(F("Performing a factory reset: "));
    if ( ! ble.factoryReset() ){
      error(F("Couldn't factory reset"));
    }
  }


  /* Disable command echo from Bluefruit */
  ble.echo(false);

  Serial.println("Requesting Bluefruit info:");
  /* Print Bluefruit information */
  ble.info();

  Serial.println(F("Please use Adafruit Bluefruit LE app to connect in Controller mode"));
  Serial.println(F("Then activate/use the sensors, color picker, game controller, etc!"));
  Serial.println();

  ble.verbose(false);  // debug info is no longer needed.

}//End Setup

/**************************************************************************/
/*!
    @brief  Constantly poll for new command or response data
*/
/**************************************************************************/
void loop(void)
{
  

  StandbyLights(2);

  
  
}//End loop


void motor(){
  colorWipe(ring.Color(0, 255, 0), 50); //Turns LED ring to Green
  
  myservo.attach(3);                 //Attach servo for use
  
  for(pos = 0; pos < ROTATE; pos++)  // goes from 0 degrees to value set in ROTATE 
  {                                  // in steps of 1 degree 
    myservo.write(180 - pos);              // tell servo to go to position in variable 'pos' 
    delay(30);                       // waits 15ms for the servo to reach the position 
  } 
  for(pos = ROTATE; pos>=0; pos--)   //goes from value set in ROTATE to 0 degrees
  {                                
    myservo.write(180 - pos);              // tell servo to go to position in variable 'pos' 
    delay(30);                       // waits 15ms for the servo to reach the position 
  } 
  myservo.detach();                  //Detach Servo Object to decrease power use
  delay(500);
}


void colorWipe(uint32_t c, uint8_t wait) {
  for(uint16_t i=0; i<ring.numPixels(); i++) {
    ring.setPixelColor(i, c);
    ring.show();
    delay(wait);
  }
}

uint32_t Wheel(byte WheelPos) {
  WheelPos = 255 - WheelPos;
  if(WheelPos < 85) {
    return ring.Color(255 - WheelPos * 3, 0, WheelPos * 3);
  }
  if(WheelPos < 170) {
    WheelPos -= 85;
    return ring.Color(0, WheelPos * 3, 255 - WheelPos * 3);
  }
  WheelPos -= 170;
  return ring.Color(WheelPos * 3, 255 - WheelPos * 3, 0);
}

void StandbyLights(uint8_t wait) {
  uint16_t i, j;

  for(j=0; j<256*5; j++) { // 5 cycles of all colors on wheel
    for(i=0; i< ring.numPixels(); i++) {
      ring.setPixelColor(i, Wheel(((i * 256 / ring.numPixels()) + j) & 255));
    }
    ring.show();
    scan();
    BLE();
    delay(wait);
  }
}


void scan(){
  delay(10);                     // Wait 50ms between pings (about 20 pings/sec). 29ms should be the shortest delay between pings.
  Serial.print("Ping: ");
  Serial.print(sonar.ping_cm()); // Send ping, get distance in cm and print result (0 = outside set distance range)
  Serial.println("cm");

  if(sonar.ping_cm() < 10 && sonar.ping_cm() != 0){
    Serial.println("Moving");
    motor();
  }
}


void BLE(){
  if(ble.isConnected()){
    Serial.println(F("******************************"));

    // Set Bluefruit to DATA mode
    Serial.println( F("Switching to DATA mode!") );
    ble.setMode(BLUEFRUIT_MODE_DATA);
  
    Serial.println(F("******************************"));
  }

  while(ble.isConnected()){
    
    /* Wait for new data to arrive */
    uint8_t len = readPacket(&ble, BLE_READPACKET_TIMEOUT);
    if (len == 0) return;
  
    /* Got a packet! */
    // printHex(packetbuffer, len);
  
  
    //Triggers a flush when a button in the Bluetooth Device is pressed
    if (packetbuffer[1] == 'B') {
      uint8_t buttnum = packetbuffer[2] - '0';
      boolean pressed = packetbuffer[3] - '0';
      Serial.print ("Poop "); 
      if (pressed) {
        Serial.println(" pressed");
        motor();
      } else {
        Serial.println(" released");
      }
    }
  }//End Ble Connection
}



